@@ -1,9 +1,10 @@
 """Regression models"""
-from sklearn import linear_model, base
-from scipy import optimize
-import numpy as np
 import copy
-from typing import Tuple, Optional, Callable
+from typing import Callable, Optional, Tuple
+
+import numpy as np
+from scipy import optimize
+from sklearn import base, linear_model
 
 
 def hinge_function(x: np.ndarray, a: float, k: float, q: float) -> np.ndarray:
@@ -20,16 +21,12 @@ def hinge_function(x: np.ndarray, a: float, k: float, q: float) -> np.ndarray:
   return k * np.minimum(x, a) + q
 
 
-coeff_init_type: type = Callable[
-  [np.ndarray, np.ndarray, float, float],
-  Tuple[float, float, float]
-]
+coeff_init_type: type = Callable[[np.ndarray, np.ndarray, float, float],
+                                 Tuple[float, float, float]]
 
-
-bounds_fun_type: type = Callable[
-  [np.ndarray, np.ndarray, float, float],
-  Tuple[Tuple[float, float, float], Tuple[float, float, float]]
-]
+bounds_fun_type: type = Callable[[np.ndarray, np.ndarray, float, float],
+                                 Tuple[Tuple[float, float, float],
+                                       Tuple[float, float, float]]]
 
 
 class HingeRegression(base.RegressorMixin, base.BaseEstimator):
@@ -64,14 +61,15 @@ class HingeRegression(base.RegressorMixin, base.BaseEstimator):
       via their individual properties
     result_ (OptimizeResult): Optimization result of the nonlinear least
       squares procedure"""
+
   def __init__(
-    self,
-    linear_regressor=linear_model.LinearRegression(),
-    linear_regressor_k: str = "coef_",
-    linear_regressor_q: str = "intercept_",
-    method: str = "dogbox",
-    coeffs_init: Optional[coeff_init_type] = None,
-    bounds: Optional[bounds_fun_type] = None,
+      self,
+      linear_regressor=linear_model.LinearRegression(),
+      linear_regressor_k: str = "coef_",
+      linear_regressor_q: str = "intercept_",
+      method: str = "dogbox",
+      coeffs_init: Optional[coeff_init_type] = None,
+      bounds: Optional[bounds_fun_type] = None,
   ):
     self.linear_regressor = linear_regressor
     self.linear_regressor_k = linear_regressor_k
@@ -90,11 +88,10 @@ class HingeRegression(base.RegressorMixin, base.BaseEstimator):
 
   @staticmethod
   def _default_coeffs_init(
-    x: np.ndarray,
-    y: np.ndarray,  # pylint: disable=W0613
-    k: float,
-    q: float
-  ) -> Tuple[float, float, float]:
+      x: np.ndarray,
+      y: np.ndarray,  # pylint: disable=W0613
+      k: float,
+      q: float) -> Tuple[float, float, float]:
     """Default coefficient initializer
 
     Args:
@@ -121,11 +118,11 @@ class HingeRegression(base.RegressorMixin, base.BaseEstimator):
     return self.coeffs_init
 
   def _default_bounds(
-    self,
-    x: np.ndarray,
-    y: np.ndarray,
-    k: float,
-    q: float  # pylint: disable=W0613
+      self,
+      x: np.ndarray,
+      y: np.ndarray,
+      k: float,
+      q: float  # pylint: disable=W0613
   ) -> Tuple[Tuple[float, float, float], Tuple[float, float, float]]:
     """Default boundaries
 
@@ -138,11 +135,31 @@ class HingeRegression(base.RegressorMixin, base.BaseEstimator):
     Returns:
       ((float, float, float), (float, float, float)): Minimum and maximum bounds
         for a, k and q"""
-    dq = np.max(np.abs(y - q))
-    return (
-      (np.min(x), 4 * k if k < 0 else k, q - dq),
-      (np.max(x), k if k < 0 else 4 * k, q + dq)
-    )
+    # Knee point bounds
+    if len(x) <= 1:
+      raise ValueError(f"Got a track of length={len(x)}. "
+                       "Consider increasing the minimum sine length")
+    else:
+      a_min = np.min(x)
+      a_max = np.max(x)
+    if a_min == a_max:
+      a_min -= 1
+      a_max += 1
+
+    # Intercept bounds
+    dq = np.max(np.abs(y - q)) or 1
+    q_min = q - dq
+    q_max = q + dq
+
+    # Slope bounds
+    if k == 0:
+      k_min = -1
+      k_max = 1
+    else:
+      k_min = 8 * k if k < 0 else k
+      k_max = k if k < 0 else 8 * k
+
+    return ((a_min, k_min, q_min), (a_max, k_max, q_max))
 
   @property
   def _bounds(self) -> bounds_fun_type:
@@ -166,10 +183,7 @@ class HingeRegression(base.RegressorMixin, base.BaseEstimator):
     return hinge_function(x, *self.coeffs_)
 
   @staticmethod
-  def _residual(
-    x: np.ndarray,
-    y: np.ndarray
-  ) -> Callable[..., np.ndarray]:
+  def _residual(x: np.ndarray, y: np.ndarray) -> Callable[..., np.ndarray]:
     """Function for residual computation
 
     Args:
@@ -214,18 +228,18 @@ class HingeRegression(base.RegressorMixin, base.BaseEstimator):
       HingeRegression: self"""
     self.linear_regressor.fit(x, y)
     self.coeffs_ = self._coeffs_init(  # pylint: disable=E1102 (false positive)
-      x, y,
-      np.squeeze(getattr(self.linear_regressor, self.linear_regressor_k)),
-      np.squeeze(getattr(self.linear_regressor, self.linear_regressor_q)),
+        x,
+        y,
+        np.squeeze(getattr(self.linear_regressor, self.linear_regressor_k)),
+        np.squeeze(getattr(self.linear_regressor, self.linear_regressor_q)),
     )
 
     self.result_ = optimize.least_squares(
-      fun=self._residual(x, y),
-      x0=self.coeffs_,
-      bounds=self._bounds(x, y, *self.coeffs_[1:]),  # pylint: disable=E1102 (false positive)
-      method=self.method,
-      **kwargs
-    )
+        fun=self._residual(x, y),
+        x0=self.coeffs_,
+        bounds=self._bounds(x, y, *self.coeffs_[1:]),  # pylint: disable=E1102 (false positive)
+        method=self.method,
+        **kwargs)
     self.coeffs_ = self.result_.x
 
     return self
