@@ -1,10 +1,12 @@
 """Sinusoidal model"""
-from sample.sms import dsp
-import numpy as np
-from sklearn import base
 import functools
 import itertools
-from typing import Optional, Tuple, Generator, Iterable, Callable, Any, List, Dict
+from typing import (Any, Callable, Dict, Generator, Iterable, List, Optional,
+                    Tuple)
+
+import numpy as np
+from sample.sms import dsp
+from sklearn import base
 
 
 def min_key(it: Iterable, key: Callable) -> Tuple[Any, Any]:
@@ -37,12 +39,13 @@ class SineTracker:
 
   Attributes:
     tracks_ (list of dict): Deactivated tracks"""
+
   def __init__(
-    self,
-    max_n_sines: int,
-    min_sine_dur: float,
-    freq_dev_offset: float,
-    freq_dev_slope: float,
+      self,
+      max_n_sines: int,
+      min_sine_dur: float,
+      freq_dev_offset: float,
+      freq_dev_slope: float,
   ):
     self.max_n_sines = max_n_sines
     self.min_sine_dur = min_sine_dur
@@ -71,9 +74,8 @@ class SineTracker:
     """All deactivated tracks in :attr:`tracks_` and those active tracks
     that would pass the cleanness check at the current state of the tracker"""
     return itertools.chain(
-      self.tracks_,
-      filter(self.track_ok, map(self.numpy_track, self._active_tracks))
-    )
+        self.tracks_,
+        filter(self.track_ok, map(self.numpy_track, self._active_tracks)))
 
   def df(self, f: float) -> float:
     """Frequency deviation threshold at given frequency
@@ -94,10 +96,7 @@ class SineTracker:
 
     Returns:
       dict: Converted track"""
-    return {
-      k: np.array(v)
-      for k, v in track.items()
-    }
+    return {k: np.array(v) for k, v in track.items()}
 
   def track_ok(self, track: dict) -> bool:
     """Check if deactivated track is ok to be saved
@@ -107,7 +106,7 @@ class SineTracker:
 
     Returns:
       bool: Whether the track is ok or not"""
-    return len(track["freq"]) > self.min_sine_dur
+    return len(track["freq"]) >= self.min_sine_dur
 
   def deactivate(self, track_index: int) -> dict:
     """Remove track from list of active tracks and save it in
@@ -142,10 +141,10 @@ class SineTracker:
       if not any(free_track):
         break
       t_i, df = min_key(
-        # choose amongst free tracks only
-        filter(free_track.__getitem__, range(self.n_active_tracks)),
-        # absolute difference from last peak's frequency
-        lambda i, p=p_i: np.abs(self._active_tracks[i]["freq"][-1] - pfreq[p])
+          # choose amongst free tracks only
+          filter(free_track.__getitem__, range(self.n_active_tracks)),
+          # absolute difference from last peak's frequency
+          lambda i, p=p_i: np.abs(self._active_tracks[i]["freq"][-1] - pfreq[p])
       )
       # If deviation is below threshold, add peak to active track
       if df < self.df(pfreq[p_i]):
@@ -156,10 +155,8 @@ class SineTracker:
         free_peak[p_i] = False
 
     # Deactivate non-continued tracks
-    for t_i in filter(
-      free_track.__getitem__,
-      reversed(range(self.n_active_tracks))
-    ):
+    for t_i in filter(free_track.__getitem__,
+                      reversed(range(self.n_active_tracks))):
       self.deactivate(t_i)
 
     # Activate new tracks for free peaks
@@ -167,10 +164,10 @@ class SineTracker:
       if self.n_active_tracks >= self.max_n_sines:
         break
       self._active_tracks.append({
-        "start_frame": self._frame,
-        "freq": [pfreq[p_i]],
-        "mag": [pmag[p_i]],
-        "phase": [pph[p_i]],
+          "start_frame": self._frame,
+          "freq": [pfreq[p_i]],
+          "mag": [pmag[p_i]],
+          "phase": [pph[p_i]],
       })
 
     self._frame += 1
@@ -190,10 +187,17 @@ class SinusoidalModel(base.TransformerMixin, base.BaseEstimator):
     max_n_sines (int): Maximum number of tracks per frame. Defaults to 100
     min_sine_dur (float): Minimum duration of a track in seconds.
       Defaults to 0.04
+    safe_sine_len (int): Minimum safe length of a track in number of
+      frames. This mainly serves as a check over the :data:`min_sine_dur`
+      parameter. If :data:`None` (default), then, do not
+      check :data:`min_sine_dur`. Set this to :data:`2` to avoid length
+      errors for the regressor.
     freq_dev_offset (float): Frequency deviation threshold at 0Hz.
       Defaults to 20
     freq_dev_slope (float): Slope of frequency deviation threshold.
       Defaults to 0.01
+    reverse (bool): Whether to process audio in reverse.
+      Defaults to False
     sine_tracker_cls (type): Sine tracker class
     save_intermediate (bool): If True, save intermediate data structures in
       the attribute :attr:`intermediate_`. Defaults to False
@@ -201,19 +205,22 @@ class SinusoidalModel(base.TransformerMixin, base.BaseEstimator):
   Attributes:
     w_ (array): Effective analysis window
     intermediate_ (dict): Dictionary of intermediate data structures"""
+
   def __init__(
-    self,
-    fs: int = 44100,
-    w: Optional[np.ndarray] = None,
-    n: int = 2048,
-    h: int = 500,
-    t: float = -90,
-    max_n_sines: int = 100,
-    min_sine_dur: float = 0.04,
-    freq_dev_offset: float = 20,
-    freq_dev_slope: float = 0.01,
-    sine_tracker_cls: type = SineTracker,
-    save_intermediate: bool = False,
+      self,
+      fs: int = 44100,
+      w: Optional[np.ndarray] = None,
+      n: int = 2048,
+      h: int = 500,
+      t: float = -90,
+      max_n_sines: int = 100,
+      min_sine_dur: float = 0.04,
+      safe_sine_len: Optional[int] = None,
+      freq_dev_offset: float = 20,
+      freq_dev_slope: float = 0.01,
+      reverse: bool = False,
+      sine_tracker_cls: type = SineTracker,
+      save_intermediate: bool = False,
   ):
     self.fs = fs
     self.w = w
@@ -222,17 +229,18 @@ class SinusoidalModel(base.TransformerMixin, base.BaseEstimator):
     self.t = t
     self.max_n_sines = max_n_sines
     self.min_sine_dur = min_sine_dur
+    self.safe_sine_len = safe_sine_len
     self.freq_dev_offset = freq_dev_offset
     self.freq_dev_slope = freq_dev_slope
+    self.reverse = reverse
     self.sine_tracker_cls = sine_tracker_cls
     self.save_intermediate = save_intermediate
 
   def fit(
-    self,
-    x: np.ndarray,
-    y=None,  # pylint: disable=W0613
-    **kwargs
-  ):
+      self,
+      x: np.ndarray,
+      y=None,  # pylint: disable=W0613
+      **kwargs):
     """Analyze audio data
 
     Args:
@@ -247,18 +255,14 @@ class SinusoidalModel(base.TransformerMixin, base.BaseEstimator):
       del self.intermediate_
     self.set_params(**kwargs)
     self.w_ = self.normalized_window
-    self.sine_tracker_ = self.sine_tracker_cls(
-      **self.sine_tracker_kwargs
-    )
+    self.sine_tracker_ = self.sine_tracker_cls(**self.sine_tracker_kwargs)
+    if self.reverse:
+      x = np.flip(x)
 
-    for mx, px in map(
-      functools.partial(self.intermediate, "stft"),
-      self.dft_frames(x)
-    ):
+    for mx, px in map(functools.partial(self.intermediate, "stft"),
+                      self.dft_frames(x)):
       ploc, pmag, pph = self.intermediate(
-        "peaks",
-        dsp.peak_detect_interp(mx, px, self.t)
-      )
+          "peaks", dsp.peak_detect_interp(mx, px, self.t))
       pfreq = ploc * self.fs / self.n  # indices to frequencies in Hz
       self.sine_tracker_(pfreq, pmag, pph)
     return self
@@ -271,11 +275,14 @@ class SinusoidalModel(base.TransformerMixin, base.BaseEstimator):
   @property
   def sine_tracker_kwargs(self) -> dict:
     """Arguments for sine tracker initialization"""
+    min_sine_len = int(self.min_sine_dur * self.fs / self.h)
+    if self.safe_sine_len is not None:
+      min_sine_len = max(self.safe_sine_len, min_sine_len)
     return dict(
-      max_n_sines=self.max_n_sines,
-      min_sine_dur=self.min_sine_dur * self.fs / self.h,
-      freq_dev_offset=self.freq_dev_offset,
-      freq_dev_slope=self.freq_dev_slope,
+        max_n_sines=self.max_n_sines,
+        min_sine_dur=min_sine_len,
+        freq_dev_offset=self.freq_dev_offset,
+        freq_dev_slope=self.freq_dev_slope,
     )
 
   def intermediate(self, key: str, value):
@@ -333,10 +340,8 @@ class SinusoidalModel(base.TransformerMixin, base.BaseEstimator):
     for i in range(a, y.size - a, self.h):
       yield y[(i - a):(i - a + self.w_.size)]
 
-  def dft_frames(
-    self,
-    x: np.ndarray
-  ) -> Iterable[Tuple[np.ndarray, np.ndarray]]:
+  def dft_frames(self,
+                 x: np.ndarray) -> Iterable[Tuple[np.ndarray, np.ndarray]]:
     """Iterable of DFT frames for a given input
 
     Args:
@@ -345,7 +350,5 @@ class SinusoidalModel(base.TransformerMixin, base.BaseEstimator):
     Returns:
       iterable: Iterable of overlapping DFT frames (magnitude and phase)
       of the padded input"""
-    return map(
-      functools.partial(dsp.dft, w=self.w_, n=self.n),
-      self.time_frames(x)
-    )
+    return map(functools.partial(dsp.dft, w=self.w_, n=self.n),
+               self.time_frames(x))
